--[[
  ****************************************************************
	Scrolling Combat Text v3.53

	Author: Grayhoof
	****************************************************************
	Description:
		A fairly simple but very configurable mod that adds damage, 
		heals, and events (dodge, parry, etc...) as 
		scrolling text above you character model, much like what 
		already happens above your target. This makes it so you do not 
		have to watch (or use) your regular combat chat window and 
		gives it a "Final Fantasy" feel.

	Thank you to:
		Nulli, for the original healthbar mod
		Kaitlin, for Mini-Group code examples
		Dhargo/Mairelon, for FlexBar code examples
		Ayny, for the new parsing code samples
		wT, for the new class based event filters.

	Official Site:
		http://rjhaney.pair.com/sct/ 
	
	****************************************************************]]

-- global flags/vars
SCTPlayer = nil;
SCT_LastHPPercent = 0;
SCT_LastTargetHPPercent = 0;
SCT_LastManaPercent = 0;
SCT_Vars_Loaded = false;
SCT_PlayerName = nil;

-- local constants
local SCT_CRIT_FADEINTIME = 0.5;
local SCT_CRIT_HOLDTIME = 2.0;
local SCT_CRIT_FADEOUTTIME = 0.5;
local SCT_CRIT_X_OFFSET = 100;
local SCT_CRIT_Y_OFFSET = 75;
local SCT_CRIT_SIZE_PERCENT = 1.25;
local SCT_CRUSH_SIZE_PERCENT = 1.15;
local SCT_GLANCE_SIZE_PERCENT = .85;
local SCT_MAX_SPEED = .025;
local SCT_MIN_UPDATE_SPEED = .01;
local SCT_TOP_POINT = 210;
local SCT_BOTTOM_POINT = 60;
local SCT_SIDE_POINT = 210;
local SCT_MIDDLE_POINT = 0;
local SCT_MAX_DISTANCE = 150;
local SCT_DIRECTION = 1;

--Animation System variables
local SCT_LASTBAR = 1;					-- Holds what the next avalible fontstring for the animation system is
local SCT_LASTCRITBAR = 1;					-- Holds what the next avalible fontstring for the animation system is
local arrAniData = {
		["aniData1"] = {},					-- These are structures that define the animation data
		["aniData2"] = {},
		["aniData3"] = {},
		["aniData4"] = {},
		["aniData5"] = {},
		["aniData6"] = {},
		["aniData7"] = {},
		["aniData8"] = {},
		["aniData9"] = {},
		["aniData10"] = {}
}

local sct_spelltypes = {
	[0] = SPELL_SCHOOL0_CAP,
	[1] = SPELL_SCHOOL1_CAP,
	[2] = SPELL_SCHOOL2_CAP,
	[3] = SPELL_SCHOOL3_CAP,
	[4] = SPELL_SCHOOL4_CAP,
	[5] = SPELL_SCHOOL5_CAP,
	[6] = SPELL_SCHOOL6_CAP,
}

local default_config = {
		["VERSION"] = SCT_Version,
		["ENABLED"] = 1,
		["ANIMATIONSPEED"] = 0.015,
		["MOVEMENT"] = 2,
		["TEXTSIZE"] = 24,
		["LOWHP"] = .4,
		["LOWMANA"] = .4,
		["SHOWHIT"] = 1,
		["SHOWMISS"] = 1,
		["SHOWDODGE"] = 1,
		["SHOWPARRY"] = 1,
		["SHOWBLOCK"] = 1,
		["SHOWSPELL"] = 1,
		["SHOWHEAL"] = 1,
		["SHOWRESIST"] = 1,
		["SHOWDEBUFF"] = 1,
		["SHOWBUFF"] = 1,
		["SHOWFADE"] = 0,
		["SHOWABSORB"] = 1,
		["SHOWLOWHP"] = 1,
		["SHOWLOWMANA"] = 1,
		["SHOWPOWER"] = 1,
		["SHOWCOMBAT"] = 0,
		["SHOWCOMBOPOINTS"] = 0,
		["SHOWHONOR"] = 1,
		["SHOWEXECUTE"] = 1,
		["SHOWREP"] = 1,
		["SHOWASMESSAGE"] = 0,
		["SHOWSELF"] = 0,
		["DIRECTION"] = 0,
		["STICKYCRIT"] = 1,
		["SPELLTYPE"] = 0,
		["ALPHA"] = 1,
		["ANITYPE"] = 1,
		["ANISIDETYPE"] = 1,
		["XOFFSET"] = 0,
		["YOFFSET"] = 0,
		["FONT"] = 1,
		["FONTSHADOW"] = 1,
		["COLORS"] = {
			[1] =  {r = 1.0, g = 0.0, b = 0.0},
			[2] =  {r = 0.0, g = 0.0, b = 1.0},
			[3] =  {r = 0.0, g = 0.0, b = 1.0},
			[4] =  {r = 0.0, g = 0.0, b = 1.0},
			[5] =  {r = 0.0, g = 0.0, b = 1.0},
			[6] =  {r = 0.5, g = 0.0, b = 0.5},
			[7] =  {r = 0.0, g = 1.0, b = 0.0},
			[8] =  {r = 0.5, g = 0.0, b = 0.5},
			[9] =  {r = 0.0, g = 0.5, b = 0.5},
			[10] =  {r = 1.0, g = 1.0, b = 0.0},
			[11] =  {r = 1.0, g = 0.5, b = 0.5},
			[12] =  {r = 0.5, g = 0.5, b = 1.0},
			[13] =  {r = 1.0, g = 1.0, b = 0.0},
			[14] =  {r = 1.0, g = 1.0, b = 1.0},
			[15] =  {r = 1.0, g = 0.5, b = 0.0},
			[16] =  {r = 0.5, g = 0.5, b = 0.7},
			[17] =  {r = 0.7, g = 0.7, b = 0.0},
			[18] =  {r = 0.7, g = 0.7, b = 0.0},
			[19] =  {r = 0.7, g = 0.7, b = 0.7},
			[20] =  {r = 0.5, g = 0.5, b = 1}
		},
		["CRITS"] = {
			[1] =  0,
			[2] =  0,
			[3] =  0,
			[4] =  0,
			[5] =  0,
			[6] =  0,
			[7] =  0,
			[8] =  0,
			[9] =  0,
			[10] =  0,
			[11] =  0,
			[12] =  0,
			[13] =  0,
			[14] =  0,
			[15] =  0,
			[16] =  0,
			[17] =  0,
			[18] =  0,
			[19] =  1,
			[20] =  0
		}
	};


----------------------
--Called when its loaded
function SCT_OnLoad()
	-- Register Startup Events
	this:RegisterEvent("VARIABLES_LOADED");
	this:RegisterEvent("ADDON_LOADED");
end

----------------------
-- Show the Option Menu
function SCT_showMenu()
	PlaySound("igMainMenuOpen");
	ShowUIPanel(SCTOptions);
end

----------------------
--Hide the Option Menu
function SCT_hideMenu()
	PlaySound("igMainMenuClose");
	SCT_EXAMPLETEXT:Hide();
	SCTOptions_SaveLoadFrame:Hide();
	HideUIPanel(SCTOptions);
	
	-- myAddOns support
	if(MYADDONS_ACTIVE_OPTIONSFRAME == SCTOptions) then
		ShowUIPanel(myAddOnsFrame);
	end
end


----------------------
--Get a clean config
function SCT_FreshVar()
	SCT_CONFIG = {};
end

----------------------
--Set the global player config
function SCT_Initialize()
		
	SCT_Chat_Message(SCT_STARTUP);
	
	-- Register Main Events
	this:RegisterEvent("CHAT_MSG_COMBAT_CREATURE_VS_SELF_MISSES");
	this:RegisterEvent("CHAT_MSG_COMBAT_CREATURE_VS_SELF_HITS");
	this:RegisterEvent("CHAT_MSG_SPELL_PERIODIC_SELF_DAMAGE");
	this:RegisterEvent("CHAT_MSG_SPELL_PERIODIC_SELF_BUFFS");
	this:RegisterEvent("CHAT_MSG_SPELL_SELF_BUFF");
	this:RegisterEvent("CHAT_MSG_COMBAT_HOSTILEPLAYER_MISSES");
	this:RegisterEvent("CHAT_MSG_COMBAT_HOSTILEPLAYER_HITS");
	this:RegisterEvent("UNIT_HEALTH");
	this:RegisterEvent("UNIT_MANA");
	this:RegisterEvent("UNIT_COMBAT");
	this:RegisterEvent("UNIT_SPELLMISS");
	this:RegisterEvent("PLAYER_REGEN_ENABLED");
	this:RegisterEvent("PLAYER_REGEN_DISABLED");
	this:RegisterEvent("PLAYER_COMBO_POINTS");
	this:RegisterEvent("CHAT_MSG_COMBAT_HONOR_GAIN");
	this:RegisterEvent("CHAT_MSG_COMBAT_SELF_HITS");
	this:RegisterEvent("CHAT_MSG_COMBAT_SELF_MISSES");
	this:RegisterEvent("CHAT_MSG_SPELL_SELF_DAMAGE");
	this:RegisterEvent("CHAT_MSG_SPELL_AURA_GONE_SELF");
	this:RegisterEvent("CHAT_MSG_SPELL_PERIODIC_CREATURE_DAMAGE");
	this:RegisterEvent("CHAT_MSG_SPELL_PERIODIC_HOSTILEPLAYER_DAMAGE");
	this:RegisterEvent("CHAT_MSG_SPELL_HOSTILEPLAYER_BUFF");
	this:RegisterEvent("CHAT_MSG_COMBAT_FACTION_CHANGE");
	this:RegisterEvent("PLAYER_TARGET_CHANGED");


	-- Add Slash Commands
	SlashCmdList["SCT"] = SCT_showMenu;
	SLASH_SCT1 = "/sct";
	
	SlashCmdList["SCTMENU"] = SCT_showMenu;
	SLASH_SCTMENU1 = "/sctmenu";
	
	SlashCmdList["SCTRESET"] = SCT_Reset;
	SLASH_SCTRESET1 = "/sctreset";
	
	SlashCmdList["SCTDISPLAY"] = SCT_CmdDisplay;
	SLASH_SCTDISPLAY1 = "/sctdisplay";
	
	-- myAddOns support
	if(myAddOnsList) then
		myAddOnsList.SCT = {
			name = SCT_CB_NAME,
			description = SCT_CB_LONG_DESC,
			version = SCT_Version,
			frame = "SCT_HEALTHTEXT",
			optionsframe = "SCTOptions",
			category = MYADDONS_CATEGORY_COMBAT
		};
	end
	
	--Cosmos support
	if ( EarthFeature_AddButton ) then
		EarthFeature_AddButton(
		   {
		      id="SCT";
		      name=SCT_CB_NAME;
		      text=SCT_CB_NAME;
		      subtext=SCT_CB_SHORT_DESC;
		      helptext=SCT_CB_LONG_DESC;
		      icon=SCT_CB_ICON;
		      callback=SCT_showMenu;
		   }
		);
	elseif (Cosmos_RegisterButton) then
		Cosmos_RegisterButton (
		   SCT_CB_NAME,
		   SCT_CB_SHORT_DESC,
		   SCT_CB_LONG_DESC,
		   SCT_CB_ICON,
		   SCT_showMenu,
		   function()
		      return true;
		   end
		);
		default_config.ENABLED = 0;
	end

	
	-- Add my options frame to the global UI panel list
	UIPanelWindows["SCTOptions"] = {area = "center", pushable = 0};

	--if there's not a config loaded
	if ( SCT_CONFIG == nil) then
		--get a new clean one
		SCT_FreshVar();
	end
	
	--set player name
	SCT_PlayerName = UnitName("player").." of "..GetCVar("realmName");
	
	--set the currentplayer config
	SCTPlayer = SCT_Config_GetPlayer();
	
	--setup animations
	SCT_aniInit();
end

----------------------
--Get or Create a config based on the current player
function SCT_Config_GetPlayer()
	if (SCT_CONFIG[SCT_PlayerName] == nil) then
		SCT_Config_NewPlayer(SCT_PlayerName);
	end
	return SCT_CONFIG[SCT_PlayerName];
end

----------------------
--Set up a default config
function SCT_Config_NewPlayer(PlayerName)
	SCT_CONFIG[PlayerName] = SCT_clone(default_config);
	local yellow = {r = 1.0, g = 1.0, b = 0.0};
	SCT_Chat_Message(SCT_PROFILE_NEW..PlayerName);
end

----------------------
--Copy a whole table
function SCT_clone(t)             -- return a copy of the table t
  local new = {};             -- create a new table
  local i, v = next(t, nil);  -- i is an index of t, v = t[i]
  while i do
  	if type(v)=="table" then 
  		v=SCT_clone(v);
  	end 
    new[i] = v;
    i, v = next(t, i);        -- get next index
  end
  return new;
end

------------------------------
---Sort a table by its keys.
---stolen from http://www.lua.org/pil/19.3.html
function SCT_pairsByKeys (t, f)
  local a = {}
  for n in pairs(t) do table.insert(a, n) end
  table.sort(a, f)
  local i = 0      -- iterator variable
  local iter = function ()   -- iterator function
    i = i + 1
    if a[i] == nil then return nil
    else return a[i], t[a[i]]
    end
  end
  return iter
end

----------------------
--Reset everything to default
function SCT_Reset()
	SCT_CONFIG[SCT_PlayerName] = nil;
	SCTPlayer = SCT_Config_GetPlayer();
	SCT_hideMenu();
	SCT_showMenu();
end

----------------------
--Get a value from player config
function SCT_Get(option)
	if (SCTPlayer ~= nil) and (SCTPlayer[option] ~= nil) then
		return SCTPlayer[option];
	else
		SCTPlayer[option] = default_config[option];
		return SCTPlayer[option];
	end
end

----------------------
--Set a value in player config
function SCT_Set(option, newVal)
	if (SCTPlayer ~= nil) then
		if ( option ) then
			SCTPlayer[option] = newVal;
		end
	end
end

----------------------
--Make sure color table exsists
function SCT_CheckColorTable(key)
	if (SCTPlayer["COLORS"] == nil) then
		SCTPlayer["COLORS"] = SCT_clone(default_config["COLORS"]);
	elseif (SCTPlayer["COLORS"][key] == nil) then
		SCTPlayer["COLORS"][key] = default_config["COLORS"][key];
	end
end

----------------------
--Get a color in the player config
function SCT_GetColor(key)
	local color = {r = 1.0, g = 1.0, b = 1.0};
	SCT_CheckColorTable(key);
	color.r = SCTPlayer["COLORS"][key].r
	color.g = SCTPlayer["COLORS"][key].g
	color.b = SCTPlayer["COLORS"][key].b
	return color;
end

----------------------
--Set a color in the player config
function SCT_SetColor(key, r, g, b)
	SCT_CheckColorTable(key);
	SCTPlayer["COLORS"][key].r = r;
	SCTPlayer["COLORS"][key].g = g;
	SCTPlayer["COLORS"][key].b = b;
end

----------------------
--Make sure crit table exsists
function SCT_CheckCritTable(key)
	if (SCTPlayer["CRITS"] == nil) then
		SCTPlayer["CRITS"] = SCT_clone(default_config["CRITS"]);
	elseif (SCTPlayer["CRITS"][key] == nil) then
		SCTPlayer["CRITS"][key] = default_config["CRITS"][key];
	end
end

----------------------
--Get a crit from player config
function SCT_GetCrit(key)	
	SCT_CheckCritTable(key);	
	return SCTPlayer["CRITS"][key];
end

----------------------
--Set a color in the player config
function SCT_SetCrit(key, value)
	SCT_CheckCritTable(key);
	SCTPlayer["CRITS"][key] = value;
end

----------------------
-- Event Handler
-- this function parses events that are printed in the combat
-- chat window then displays the health changes
function SCT_OnEvent()
	
	local critical;
		
	--Player loaded completely
	if (event == "ADDON_LOADED") then
		if (strlower(arg1) == "sct") then
			SCT_Vars_Loaded = true;
			SCT_Initialize();
		end
		return;
	end
	
	--Vars loaded, only load once (bug fix for some users)
	if (event == "VARIABLES_LOADED") then
		if (SCT_Vars_Loaded == false) then
			SCT_Vars_Loaded = true;
			SCT_Initialize();
		end
		return;
	end
	
	-- if its enabled
	if (SCT_Get("ENABLED") == 1) then
		
		--Player Health
		if (event == "UNIT_HEALTH") then
			if (arg1 == "player") and (SCT_Get("SHOWLOWHP") == 1) then
				local warnlevel = SCT_Get("LOWHP");
				local HPPercent = UnitHealth("player") / UnitHealthMax("player");
	      if (HPPercent < warnlevel) and (SCT_LastHPPercent >= warnlevel) then
	      	SCT_Display_Toggle(SCT_LowHP.." ("..UnitHealth("player")..")", SCT_GetColor(11), SCT_GetCrit(11));
	      end
	      SCT_LastHPPercent = HPPercent;
	      return;
			end
			--execute check
			
			if (arg1 == "target") then
				local currentclass = UnitClass("player");
				if (UnitIsEnemy("target", "player")) and (UnitIsDead("target")~=true) and (UnitIsCorpse("target")~=true) and (SCT_Get("SHOWEXECUTE") == 1) and ((currentclass == "Warrior") or (currentclass == "Paladin")) then
					local HPTargetPercent = UnitHealth("target");
		      if (HPTargetPercent < 20) and (SCT_LastTargetHPPercent >= 20) then
		      	local msg;
		      	if (currentclass == "Warrior") then
		      		msg = SCT_ExecuteMessage;
		      	else
		      		msg = SCT_WrathMessage;
		      	end
		      	SCT_Display_Toggle(msg, SCT_GetColor(19), SCT_GetCrit(19));
		      end
		      SCT_LastTargetHPPercent = HPTargetPercent;
		    end
	      return;
			end
			return;
			
		--Target Changed
		elseif (event == "PLAYER_TARGET_CHANGED") then
			if (UnitIsEnemy("target", "player") and (UnitIsDead("target")~=true) and (UnitIsCorpse("target")~=true)) then
				SCT_LastHPPercent = UnitHealth("target");
			else
				SCT_LastHPPercent = 100;
			end
			
		--Player Mana
		elseif (event == "UNIT_MANA") then
			if (arg1 == "player") and (SCT_Get("SHOWLOWMANA") == 1) and (UnitPowerType("player") == 0)then
				local warnlevel = SCT_Get("LOWMANA");
				local ManaPercent = UnitMana("player") / UnitManaMax("player");
	      if (ManaPercent < warnlevel) and (SCT_LastManaPercent >= warnlevel) then
	      	SCT_Display_Toggle(SCT_LowMana.." ("..UnitMana("player")..")", SCT_GetColor(12), SCT_GetCrit(12));
	      end
	      SCT_LastManaPercent = ManaPercent;
			end
			return;
			
		--Player Combat
		elseif (event =="PLAYER_REGEN_DISABLED") then
			if (SCT_Get("SHOWCOMBAT") == 1) then
				SCT_Display_Toggle(SCT_Combat, SCT_GetColor(14), SCT_GetCrit(14));	
			end
			return;
			
		--Player NoCombat
		elseif (event =="PLAYER_REGEN_ENABLED") then
			if (SCT_Get("SHOWCOMBAT") == 1) then
				SCT_Display_Toggle(SCT_NoCombat, SCT_GetColor(14), SCT_GetCrit(14));
			end
			return;
			
		--Player Combo Points
		elseif (event == "PLAYER_COMBO_POINTS") then
			if (SCT_Get("SHOWCOMBOPOINTS") == 1) then
				local SCT_CP = GetComboPoints();
				if (SCT_CP ~= 0) then
					local SCT_CP_Message = SCT_CP.." "..SCT_ComboPoint;
					if (SCT_CP == 5) then
						SCT_CP_Message = SCT_CP_Message.." "..SCT_FiveCPMessage;
					end;
					SCT_Display_Toggle(SCT_CP_Message, SCT_GetColor(15), SCT_GetCrit(15));
				end
			end
			return;
			
		--Player Honor Gain
		elseif (event == "CHAT_MSG_COMBAT_HONOR_GAIN") then
			for honor in string.gfind(arg1, SCT_HONOR_SEARCH) do
				if (SCT_Get("SHOWHONOR") == 1) then
					SCT_Display_Toggle("+"..honor.." "..SCT_Honor, SCT_GetColor(16), SCT_GetCrit(16));
				end
				return;
			end
			
		--Player Reputation Gain
		elseif (event == "CHAT_MSG_COMBAT_FACTION_CHANGE") then
			--Gain
			for faction, size, amount in string.gfind(arg1, SCT_REP_GAIN_SEARCH) do
				if (SCT_Get("SHOWREP") == 1) then
					SCT_Display_Toggle("+"..amount.." "..REPUTATION.." ("..faction..")", SCT_GetColor(20), SCT_GetCrit(20));
				end
				return;
			end
			--Lost
			for faction, size, amount in string.gfind(arg1, SCT_REP_LOST_SEARCH) do
				if (SCT_Get("SHOWREP") == 1) then
					SCT_Display_Toggle("-"..amount.." "..REPUTATION.." ("..faction..")", SCT_GetColor(20), SCT_GetCrit(20));
				end
				return;
			end
				
		--Player Misses
		elseif (event == "CHAT_MSG_COMBAT_CREATURE_VS_SELF_MISSES" or event == "CHAT_MSG_COMBAT_HOSTILEPLAYER_MISSES") then
				
			--partial Absorob
			for num, num2 in string.gfind( arg1, SCT_ABSORB_AMOUNT_SEARCH) do
				if (SCT_Get("SHOWABSORB") == 1) then
					SCT_Display_Toggle(num2, SCT_GetColor(10), SCT_GetCrit(10), SCT_YOU..SCT_Absorb.." "..num2);
				end
				return;
			end
			
			--if none, return
			return;
			
		--Player Hits
		elseif (event == "CHAT_MSG_COMBAT_CREATURE_VS_SELF_HITS" or event == "CHAT_MSG_COMBAT_HOSTILEPLAYER_HITS") then				
					
			--partial block
			for num, num2 in string.gfind( arg1, SCT_BLOCK_AMOUNT_SEARCH) do
				--extra search here cause crsuhing and glancing blows, and
				--the punctuation of the line messes up SCT_BLOCK_AMOUNT_SEARCH
				num3, num4 = string.find(num2, SCT_BLOCK_SEARCH);
				if num3 then
					if (SCT_Get("SHOWBLOCK") == 1) then
						SCT_Display_Toggle(num2, SCT_GetColor(5), SCT_GetCrit(5), SCT_YOU..SCT_BlockMsg.." "..num2);
					end
				end
				return;
			end
			
			--if none, return
			return;
			
			
		--Debuffs
		elseif (event == "CHAT_MSG_SPELL_PERIODIC_SELF_DAMAGE") then
			
			--Configured self buffs
			if (SCT_CustomEventSearch(arg1) ~= true) then
				--DoT Heal, Mana, Rage, Energy
				for type, num, name in string.gfind( arg1, SCT_YOU_GAIN_AMOUNT_SEARCH ) do
					if (strlower(name) == strlower(MANA)) or (strlower(name) == strlower(ENERGY)) or (strlower(name) == strlower(RAGE)) then
						if (SCT_Get("SHOWPOWER") == 1) then
							SCT_Display("+"..num.." "..name, SCT_GetColor(13), SCT_GetCrit(13));
						end
					end
					return;
				end
				
				--Debuff
				for num in string.gfind(arg1, SCT_DEBUFF_NAME_SEARCH) do
					if (SCT_Get("SHOWDEBUFF") == 1) then
						SCT_Display_Toggle("("..num..")", SCT_GetColor(9), SCT_GetCrit(9), SCT_AFFLICTED_BY..num);	
					end
					return;
				end
			end
			
			--if none, return
			return;
			
		--Self Buffs
		elseif (event == "CHAT_MSG_SPELL_SELF_BUFF") then
			
			--Configured self buffs
			if (SCT_CustomEventSearch(arg1) ~= true) then
				--DoT Heal, Mana, Rage, Energy
				for type, num, name in string.gfind( arg1, SCT_YOU_GAIN_AMOUNT_SEARCH ) do
					if (strlower(name) == strlower(MANA)) or (strlower(name) == strlower(ENERGY)) or (strlower(name) == strlower(RAGE)) then
						if (SCT_Get("SHOWPOWER") == 1) then
							SCT_Display("+"..num.." "..name, SCT_GetColor(13), SCT_GetCrit(13));
						end
					end
					return;
				end
				
				--Buff
				for name in string.gfind( arg1, SCT_YOU_GAIN_SEARCH ) do
					if (SCT_Get("SHOWBUFF") == 1) then
						SCT_Display_Toggle("["..name.."]", SCT_GetColor(17), SCT_GetCrit(17), SCT_YOU_GAIN..name);
					end
					return;
				end
			end
			
			--if none, return
			return;
			
		--Other Self Buffs
		elseif (event == "CHAT_MSG_SPELL_PERIODIC_SELF_BUFFS") then
		
			--Configured self buffs
			if (SCT_CustomEventSearch(arg1) ~= true) then
				--DoT Heal, Mana, Rage, Energy
				for type, num, name in string.gfind( arg1, SCT_YOU_GAIN_AMOUNT_SEARCH ) do
					if (strlower(name) == strlower(MANA)) or (strlower(name) == strlower(ENERGY)) or (strlower(name) == strlower(RAGE)) then
						if (SCT_Get("SHOWPOWER") == 1) then
							SCT_Display("+"..num.." "..name, SCT_GetColor(13), SCT_GetCrit(13));
						end
					end
					return;
				end
				
				--Buff
				for name in string.gfind( arg1, SCT_YOU_GAIN_SEARCH ) do
					if (SCT_Get("SHOWBUFF") == 1) then
						SCT_Display_Toggle("["..name.."]", SCT_GetColor(17), SCT_GetCrit(17), SCT_YOU_GAIN..name);
					end
					return;
				end
			end
			
			--if none, return
			return;
			
		--Buff Fades
		elseif (event == "CHAT_MSG_SPELL_AURA_GONE_SELF" ) then
			
			--Buff Fades
			if (SCT_CustomEventSearch(arg1) ~= true) then
				for name in string.gfind(arg1, SCT_FADE_SEARCH) do
					if (SCT_Get("SHOWFADE") == 1) then
						SCT_Display_Toggle("-["..name.."]", SCT_GetColor(18), SCT_GetCrit(18), name..SCT_YOU_FADE);	
					end
					return;
				end
			end
			
			--if none, return
			return;
			
		--Other Self Events (for searching only)
		elseif (event == "CHAT_MSG_COMBAT_SELF_HITS" or 
						event == "CHAT_MSG_COMBAT_SELF_MISSES" or 
						event == "CHAT_MSG_SPELL_SELF_DAMAGE" or 
						event == "CHAT_MSG_SPELL_AURA_GONE_SELF" or
						event == "CHAT_MSG_SPELL_PERIODIC_CREATURE_DAMAGE" or
						event == "CHAT_MSG_SPELL_PERIODIC_HOSTILEPLAYER_DAMAGE") or 
						event == "CHAT_MSG_SPELL_HOSTILEPLAYER_BUFF" then
			
			--Configured events
			SCT_CustomEventSearch(arg1);
			return;	
			
		--Combat Event
		elseif((event == "UNIT_COMBAT") and (arg1 == "player")) then
		
			--Set Crit Flag
			if(arg3 == "CRITICAL") then
				critical=1;
			else
				critical=0;
			end
			
			--If Damage
			if(arg2 == "WOUND") then
				-- If Absorb
				if(arg3 == "ABSORB") then
					if (SCT_Get("SHOWABSORB") == 1) then
						if (SCT_GetCrit(10) == 1) then
							critical = 1;
						end
						SCT_Display_Toggle(SCT_Absorb, SCT_GetColor(10), critical, SCT_YOU..SCT_Absorb);
					end
					return;	
				end
				
				--If Damage
				if(arg4 ~= 0 or arg4 ~= nil) then
					--Spell
					if (arg5 > 0) then
						if (SCT_Get("SPELLTYPE") ==1) then
							arg4 = arg4.." ("..sct_spelltypes[arg5]..")";
						end
						if (SCT_Get("SHOWSPELL") ==1) then
							if (SCT_GetCrit(6) == 1) then
								critical = 1;
							end
							SCT_Display( "-"..arg4 , SCT_GetColor(6), critical);
						end
					--Health
					else
						if (SCT_Get("SHOWHIT") == 1) then
							--see if its glacing or crushing
							if ( arg3 == "CRUSHING" ) then
								critical = 2;
							elseif ( arg3 == "GLANCING" ) then
								critical = 3;
							end
							if (SCT_GetCrit(1) == 1) then
								critical = 1;
							end
							SCT_Display("-"..arg4 , SCT_GetColor(1), critical);
						end
					end
				end
				return;
			
			--If A Heal
			elseif(arg2 == "HEAL") then
				if(arg4 ~= 0 or arg4 ~= nil) then
					if (SCT_Get("SHOWHEAL") == 1) then
						if (SCT_GetCrit(7) == 1) then
							critical = 1;
						end
						SCT_Display( "+"..arg4 , SCT_GetColor(7), critical);
					end
				end
				return;
				
			--If a Dodge
			elseif(arg2 == "DODGE") then
				if (SCT_Get("SHOWDODGE") == 1) then
					if (SCT_GetCrit(3) == 1) then
						critical = 1;
					end
					SCT_Display_Toggle(SCT_DodgeMsg, SCT_GetColor(3), critical, SCT_YOU..SCT_DodgeMsg);		
				end
				return;
			
			--If a Full Block
			elseif(arg2 == "BLOCK") then
				if (SCT_Get("SHOWBLOCK") == 1) then
					if (SCT_GetCrit(5) == 1) then
						critical = 1;
					end
					SCT_Display_Toggle(SCT_BlockMsg, SCT_GetColor(5), critical, SCT_YOU..SCT_BlockMsg);				
				end
				return;
			
			--If a Parry
			elseif(arg2 == "PARRY") then
				if (SCT_Get("SHOWPARRY") == 1) then
					if (SCT_GetCrit(4) == 1) then
						critical = 1;
					end
					SCT_Display_Toggle(SCT_ParryMsg, SCT_GetColor(4), critical, SCT_YOU..SCT_ParryMsg);		
				end
				return;
				
			--If a Miss
			elseif(arg2 == "MISS") then
				if (SCT_Get("SHOWMISS") == 1) then
					if (SCT_GetCrit(2) == 1) then
						critical = 1;
					end
					SCT_Display_Toggle(SCT_MissMsg, SCT_GetColor(2), critical, SCT_TARGET..SCT_MissMsg);			
				end
				return;
			
			--If Resist
			elseif(arg2 == "RESIST") then
				if(SCT_Get("SHOWRESIST") == 1) then
					if (SCT_GetCrit(8) == 1) then
						critical = 1;
					end
					SCT_Display_Toggle(SCT_ResistMsg, SCT_GetColor(8), critical, SCT_YOU..SCT_ResistMsg);		
				end
				return;
			end

			--if none, return
			return;
		
		--Sepll Miss events
		elseif((event == "UNIT_SPELLMISS") and (arg1 == "player")) then
				
			--If Deflected
			if(arg2 == "DEFLECTED") then
				if(SCT_Get("SHOWRESIST") == 1) then
					if (SCT_GetCrit(8) == 1) then
						critical = 1;
					end
					SCT_Display_Toggle(SCT_DeflectMsg, SCT_GetColor(8), critical, SCT_YOU..SCT_DeflectMsg);			
				end
				return;
			end
		end
	end
end

----------------------
--Find a custom capture event message
function SCT_CustomEventSearch(arg1)
	local strTempMessage;
	local currentcolor;
	local iscrit;
	local table;
	local currentclass = UnitClass("player");
	local classfound;
		
	for key, value in SCT_pairsByKeys(SCT_Event_Config) do
		--default class found to true
		classfound = true;
		
		--check if they want to see it for this class
		if (value.class) then
			--if want to filter by class, default to false
			classfound = false;
			for key2, class in value.class do
				if (strlower(currentclass) == strlower(class)) then
					classfound = true;
					break;
				end
			end
		end
		
		if (classfound) then
			for carg1, carg2, carg3, carg4, carg5 in string.gfind(arg1, value.search) do
				strTempMessage = value.name;
				--if there are capture args
				if ((value.argcount) and (value.argcount > 0) and (value.argcount < 6)) then
					table = {carg1, carg2, carg3, carg4, carg5}
					--loop though each capture arg. if it exists replace the display position with it
					for i = 1, value.argcount do
						if (table[i]) then
							strTempMessage, strCount = string.gsub(strTempMessage, "*"..i, table[i]);
						end
					end			 
				end
				--get color
				currentcolor = {r = 1.0, g = 1.0, b = 1.0};
				iscrit = 0
				if (value.r and value.g and value.b) then
					currentcolor.r = value.r;
					currentcolor.g = value.g;
					currentcolor.b = value.b;
				end
				--see if crit
				if (value.iscrit) then
					iscrit = value.iscrit
				end
				--display
				SCT_Display_Toggle(strTempMessage, currentcolor, iscrit);
				return true;				
			end
		end
	end
	return false;
end

----------------------
--Display the Text based on message flag
function SCT_Display_Toggle(msg1, color, iscrit, msg2)
	if (SCT_Get("SHOWASMESSAGE") == 1) then
		if (msg2) then
			msg1 = msg2;
		end
		SCT_Display_Message( msg1, color );
	else
		SCT_Display( msg1 , color, iscrit, "event");
	end	
end

----------------------
--Display the Text
function SCT_Display(msg, color, iscrit, type)
	local adat, curDir;
	local startpos, lastpos, textsize;
	
	--Set up  text animation
	adat = arrAniData["aniData"..SCT_LASTBAR];
	
	-- if its active (rarely happens as all 10 must be in use), set points, if not set as active
	if (adat.Active == true) then
		SCT_aniReset(adat);
		adat.Active = true;
	end
			
	--set alpha
	adat.alpha = SCT_Get("ALPHA");
		
	--If they want to tag all self events
	if (SCT_Get("SHOWSELF") == 1) then
		msg = SCT_SelfFlag..msg..SCT_SelfFlag
	end
		
	--If its a crit hit, increase the size
	if (iscrit == 1) then
		textsize = SCT_Get("TEXTSIZE") * SCT_CRIT_SIZE_PERCENT;
		if (SCT_Get("STICKYCRIT") == 1) then
			adat.crit = true;
			local critcount = SCT_critCount();
			
			adat.posY = (SCT_TOP_POINT + SCT_BOTTOM_POINT)/2;
			
			--if there are other Crits active, set offset.
			if (critcount > 1) then
				--alternate left to right, up, down
				if (SCT_LASTCRITBAR == 1) then
					adat.posX = adat.posX + SCT_CRIT_X_OFFSET;
					adat.posY = adat.posY + SCT_CRIT_Y_OFFSET;
					SCT_LASTCRITBAR = 2;
				elseif (SCT_LASTCRITBAR == 2) then
					adat.posX = adat.posX + SCT_CRIT_X_OFFSET;
					adat.posY = adat.posY - SCT_CRIT_Y_OFFSET;
					SCT_LASTCRITBAR = 3;
				elseif (SCT_LASTCRITBAR == 3) then
					adat.posX = adat.posX - SCT_CRIT_X_OFFSET;
					adat.posY = adat.posY + SCT_CRIT_Y_OFFSET;
					SCT_LASTCRITBAR = 4;
				elseif (SCT_LASTCRITBAR == 4) then
					adat.posX = adat.posX - SCT_CRIT_X_OFFSET;
					adat.posY = adat.posY - SCT_CRIT_Y_OFFSET;
					SCT_LASTCRITBAR = 5;
				else
					SCT_LASTCRITBAR = 1;
				end
			else
				SCT_LASTCRITBAR = 1;
			end
		end
	--Crushing
	elseif (iscrit == 2) then
		textsize = SCT_Get("TEXTSIZE") * SCT_CRUSH_SIZE_PERCENT;
	--glancing
	elseif (iscrit == 3) then
		textsize = SCT_Get("TEXTSIZE") * SCT_GLANCE_SIZE_PERCENT;
	else
		textsize = SCT_Get("TEXTSIZE");
	end
		
	--if its not a sticky critm set up normal text
	if (adat.crit ~= true) then
		if (SCT_Get("ANITYPE") == 1) then
			--get the last known point of active items
			lastpos = SCT_MinPoint()
			
			if (SCT_Get("DIRECTION") == 0) then
				--move the position down
				if ((lastpos - adat.posY) <= SCT_Get("TEXTSIZE")) then
					adat.posY = adat.posY - (SCT_Get("TEXTSIZE") - (lastpos - adat.posY));
				end
				--if its gone too far down, stop
				if (adat.posY < (SCT_BOTTOM_POINT - SCT_MAX_DISTANCE)) then
					adat.posY = (SCT_BOTTOM_POINT - SCT_MAX_DISTANCE)
				end
			else
				--move the position up
				if ((adat.posY - lastpos) <= SCT_Get("TEXTSIZE")) then
					adat.posY = adat.posY + (SCT_Get("TEXTSIZE") - (adat.posY - lastpos));
				end
				--if its gone too far up, stop
				if (adat.posY > (SCT_TOP_POINT + SCT_MAX_DISTANCE)) then
					adat.posY = (SCT_TOP_POINT + SCT_MAX_DISTANCE)
				end
			end
		else
			--get direction type
			if (SCT_Get("ANISIDETYPE") == 1) then
				SCT_DIRECTION = SCT_DIRECTION * -1;
				curDir = SCT_DIRECTION;
			elseif (SCT_Get("ANISIDETYPE") == 2) then
				if (type=="event") then
					curDir = -1;
				else
					curDir = 1;
				end
			elseif (SCT_Get("ANISIDETYPE") == 3) then
				if (type=="event") then
					curDir = 1;
				else
					curDir = -1;
				end
			end
			
			adat.dir = curDir;
			
			--set animation start pos.
			if (SCT_Get("ANITYPE") == 2) then
				adat.addY = random(3,6);
				adat.posX = adat.posX - (20 * adat.dir);
			elseif (SCT_Get("ANITYPE") == 3) then
				adat.posX = adat.posX - (55 * adat.dir);
				adat.posY = SCT_BOTTOM_POINT + (random(0,200) - 100);
			elseif (SCT_Get("ANITYPE") == 4) then
				adat.posX = adat.posX - (20 * adat.dir);
				adat.addY = random(9,12);
				adat.addX = random(9,12);
			end
		end
	end
	
	--set to active
	adat.Active = true;
	
	--Start up onUpdate, note this is done after active is set to true
	if (not SCT_HEALTHTEXT:IsVisible()) then
		SCT_HEALTHTEXT:Show();
	end
	
	--set text size
	--adat.FObject:SetFont("Interface\\Addons\\Fonts\\Tw_Cen_MT_Bold.TTF",textsize);
	SCT_SetFontSize(adat.FObject, textsize);
	--set the color
	adat.FObject:SetTextColor(color.r, color.g, color.b);
	--set alpha
	adat.FObject:SetAlpha(adat.alpha);
	--Position
	adat.FObject:SetPoint("CENTER", "UIParent", "CENTER", adat.posX, adat.posY);
	--Set the text to display
	adat.FObject:SetText(msg);
		
	--update current text being used
	SCT_LASTBAR = SCT_LASTBAR + 1;
	
	--if we reached the end, set to first
	if (SCT_LASTBAR >= 11) then
		SCT_LASTBAR = 1;
	end
end

----------------------
--Displays a message at the top of the screen
function SCT_Display_Message(msg, color)
		SCT_MSGTEXT:AddMessage(msg, color.r, color.g, color.b, SCT_Get("ALPHA"), 1);
end

----------------------
--Displays a message to chat
function SCT_Chat_Message(msg)
	if( DEFAULT_CHAT_FRAME ) then
		DEFAULT_CHAT_FRAME:AddMessage(msg);
	end
end

----------------------
--Display text from a command line
function SCT_CmdDisplay(msg)	
	
	local message = nil;
	local colors = nil;
	
	--If there are not parameters, display useage
	if string.len(msg) == 0 then
		SCTDisplayUseage();
	--Get message anf colors if quotes used
	elseif string.sub(msg,1,1) == "'" then
		local location = string.find(string.sub(msg,2),"'")
		if location and (location>1) then
			message = string.sub(msg,2,location)
			colors = string.sub(msg,location+1);
		end
	--Get message anf colors if single word used
	else
		local idx = string.find(msg," ");
		if (idx) then
			message = string.sub(msg,1,idx-1);
			colors = string.sub(msg,idx+1);
		else
			message = msg;
		end
	end
	
	--if they sent colors, grab them
	local firsti, lasti, red, green, blue = nil;
	if (colors ~= nil) then
		firsti, lasti, red, green, blue = string.find (colors, "(%w+) (%w+) (%w+)");
	end
    
  local color = {r = 1.0, g = 1.0, b = 1.0}; 
	
	--if they sent 3 colors use them, else use default white
  if (red == nil) or (green == nil) or (blue == nil) then
  	SCT_Display(message, color);
  else
  	color.r = (tonumber(red)/10)
  	color.g = (tonumber(green)/10)
  	color.b = (tonumber(blue)/10)
  	SCT_Display(message, color);
  end
end

----------------------
--Explains how to use sctdisplay
function SCTDisplayUseage()
    SCT_Chat_Message(SCT_DISPLAY_USEAGE);
end

----------------------
-- Upate animations that are being used
function SCT_updateAnimation(elapsed)	
	local anyActive = false;
	
	for key, value in arrAniData do
		if (value.Active) then
			anyActive = true;
			SCT_doAnimation(value, elapsed);
		end
	end
		
	--if none are active, stop onUpdate;
	if ((anyActive ~= true) and (SCT_HEALTHTEXT:IsVisible())) then
		SCT_HEALTHTEXT:Hide();
	end

end

----------------------
--Move text to get the animation
function SCT_doAnimation(aniData, elapsed)
	--If a crit			
	aniData.lastupdate = aniData.lastupdate + elapsed;
	if (aniData.crit) then		
		--display crit		
		local elapsedTime = aniData.lastupdate;
		local fadeInTime = SCT_CRIT_FADEINTIME;
		if ( elapsedTime < fadeInTime ) then
			local alpha = (elapsedTime / fadeInTime);
			alpha = alpha * SCT_Get("ALPHA");
			aniData.FObject:SetAlpha(alpha);
			return;
		end
		local holdTime = (SCT_CRIT_HOLDTIME * (SCT_Get("ANIMATIONSPEED")/SCT_MAX_SPEED));
		if ( elapsedTime < (fadeInTime + holdTime) ) then
			aniData.FObject:SetAlpha(SCT_Get("ALPHA"));
			return;
		end
		local fadeOutTime = SCT_CRIT_FADEOUTTIME;
		if ( elapsedTime < (fadeInTime + holdTime + fadeOutTime) ) then
			local alpha = 1 - ((elapsedTime - holdTime - fadeInTime) / fadeOutTime);
			alpha = alpha * SCT_Get("ALPHA");
			aniData.FObject:SetAlpha(alpha);
			return;
		end
		--reset crit
		SCT_aniReset(aniData);
		
	--else normal text or event text
	elseif (SCT_Get("ANITYPE") == 1) then
		local startpos, endpos, step, alpha;
		local avgpos, diffpos;
	
		avgpos = (SCT_TOP_POINT + SCT_BOTTOM_POINT)/2;
		diffpos = (SCT_TOP_POINT - SCT_BOTTOM_POINT)/2;
		
		--if its time to update, move the text step positions
		if (aniData.lastupdate > SCT_Get("ANIMATIONSPEED")) then
			--set parameters based on direction settings
			if (SCT_Get("DIRECTION") == 0) then
				startpos = SCT_BOTTOM_POINT;
				endpos = SCT_TOP_POINT;
				step = SCT_Get("MOVEMENT");
				if ((aniData.posY) > avgpos) then
					alpha = (1 - ((aniData.posY-avgpos) / (diffpos - step)));
				else
					alpha = 1;
				end
			else
				startpos = SCT_TOP_POINT;
				endpos = SCT_BOTTOM_POINT;
				step = SCT_Get("MOVEMENT") * -1;
				if ((aniData.posY) < avgpos) then
					alpha = (1 - (((avgpos + SCT_Get("MOVEMENT")) - aniData.posY) / diffpos));
				else
					alpha = 1;
				end
			end
			
			if (alpha > 1) then
				alpha = 1;
			end

			alpha = alpha * SCT_Get("ALPHA");
					
			aniData.posY = aniData.posY + step;
			aniData.lastupdate = 0;
			aniData.alpha = alpha;
			
			aniData.FObject:SetAlpha(aniData.alpha);
			aniData.FObject:SetPoint("CENTER", "UIParent", "CENTER", aniData.posX, aniData.posY);
			
			--if it reachs the end, reset
			if (SCT_Get("DIRECTION") == 0) then
				if (aniData.posY >= endpos) then
					SCT_aniReset(aniData);
				end
			else
				if (aniData.posY <= endpos) then
					SCT_aniReset(aniData);
				end
			end
		end
	--else rainbow text
	elseif (SCT_Get("ANITYPE") == 2) then
		
		--if its time to update, move the text step positions
		if (aniData.lastupdate > SCT_Get("ANIMATIONSPEED")) then
		
			--display rainbow text			
			if (aniData.addY > 0) then
					aniData.addY = aniData.addY - 0.22;
			else
					aniData.addY = aniData.addY - 0.18;
			end
			if (aniData.addY < -7) then
					aniData.addY = -7;
			end	
			
			aniData.posY = aniData.posY + aniData.addY;
			aniData.posX = aniData.posX - 2.2 * aniData.dir;
	
			if ( aniData.posY < (SCT_BOTTOM_POINT - SCT_MAX_DISTANCE) ) then			
					aniData.alpha = aniData.alpha - 0.05;
					if (aniData.alpha < 0) then						
						aniData.alpha = 0;
					end
			end
			
			aniData.lastupdate = 0;
			aniData.FObject:SetAlpha(aniData.alpha);
			aniData.FObject:SetPoint("CENTER", "UIParent", "CENTER", aniData.posX, aniData.posY);
			
			if (aniData.alpha <= 0) then
					SCT_aniReset(aniData);
			end
		end
	--else Horizontal
	elseif (SCT_Get("ANITYPE") == 3) then
		
		--display Horizontal
		--if its time to update, move the text step positions
		if (aniData.lastupdate > SCT_Get("ANIMATIONSPEED")) then
		
			aniData.posX = aniData.posX - (SCT_Get("MOVEMENT") * aniData.dir);
	
			if ( abs(SCT_MIDDLE_POINT - aniData.posX) > (SCT_SIDE_POINT-60) ) then			
					aniData.alpha = aniData.alpha - 0.05;
					if (aniData.alpha < 0) then						
						aniData.alpha = 0;
					end
			end
			
			aniData.lastupdate = 0;
			aniData.FObject:SetAlpha(aniData.alpha);
			aniData.FObject:SetPoint("CENTER", "UIParent", "CENTER", aniData.posX, aniData.posY);
			
			if (aniData.alpha <= 0) then
					SCT_aniReset(aniData);
			end
		end
								
	--else Angled Down
	elseif (SCT_Get("ANITYPE") == 4) then
		
		--if its time to update, move the text step positions
		if (aniData.lastupdate > SCT_Get("ANIMATIONSPEED")) then
					
			if ( abs(SCT_MIDDLE_POINT - aniData.posX) < (SCT_SIDE_POINT - 60) and (aniData.delay <= 35) ) then			
					aniData.posY = aniData.posY - aniData.addY;				
					aniData.posX = aniData.posX - aniData.addX * aniData.dir;			
			elseif (aniData.delay <= 20) then
					aniData.delay = aniData.delay + 1;
					aniData.posY = aniData.posY + (random(0,70) - 35) * 0.02;
					aniData.posX = aniData.posX + (random(0,70) - 35) * 0.02;
			elseif (aniData.delay <= 35) then
					aniData.delay = aniData.delay + 1;
			else
					aniData.posY = aniData.posY + SCT_Get("MOVEMENT");
					aniData.posX = aniData.posX - SCT_Get("MOVEMENT") * aniData.dir;
					aniData.alpha = aniData.alpha - 0.02;
					if (aniData.alpha < 0) then						
						aniData.alpha = 0;
					end
			end
			
			aniData.lastupdate = 0;
			aniData.FObject:SetAlpha(aniData.alpha);
			aniData.FObject:SetPoint("CENTER", "UIParent", "CENTER", aniData.posX, aniData.posY);
			
			if (aniData.alpha <= 0) then
					SCT_aniReset(aniData);
			end
		end	
	end
end

----------------------
--count the number of crits active
function SCT_critCount()
	local count = 0
	
	for key, value in arrAniData do
		if (value.crit) then
			count = count + 1;
		end
	end
	
	return count;
end

----------------------
--get the min current min point
function SCT_MinPoint()	
	
	local posY;
	
	if (SCT_Get("DIRECTION") == 0) then
		posY = SCT_TOP_POINT;
		for key, value in arrAniData do
			if ((value.Active) and (value.posY < posY) and (not value.crit)) then
				posY = value.posY;
			end
		end
	else
		posY = SCT_BOTTOM_POINT;
		for key, value in arrAniData do
			if ((value.Active) and (value.posY > posY) and (not value.crit)) then
				posY = value.posY;
			end
		end
	end
	
	return posY;
end

----------------------
--Rest the text animation
function SCT_aniReset(aniData)	
	local startpos;
	if ((SCT_Get("DIRECTION") == 1) and (SCT_Get("ANITYPE") == 1)) then
		startpos = SCT_TOP_POINT;
	else
		startpos = SCT_BOTTOM_POINT;
	end
	
	aniData.Active = false;
	aniData.crit = false;
	aniData.posY = startpos;
	aniData.posX = SCT_MIDDLE_POINT;
	aniData.addY = 0;
	aniData.addX = 0;
	aniData.alpha = 0;
	aniData.lastupdate = 0;
	aniData.dir = 1;
	aniData.delay = 0;
	
	aniData.FObject:SetAlpha(aniData.alpha);
	aniData.FObject:SetPoint("CENTER", "UIParent", "CENTER", aniData.posX, aniData.posY);
	
end

----------------------
--Rest all the text animations
function SCT_aniResetAll()
	
	for key, value in arrAniData do
		SCT_aniReset(value);
		--set font only on global update
		SCT_SetFont(value.FObject);
	end
	
end

----------------------
--Initial animation settings
function SCT_aniInit()

	for key, value in arrAniData do
		value.FObject = getglobal("SCT"..key);
	end
	
	SCT_UpdateGlobalPos();
end

------------------------
--Update the global position vars
function SCT_UpdateGlobalPos()
	SCT_TOP_POINT = SCT_Get("YOFFSET") + SCT_MAX_DISTANCE;
	SCT_BOTTOM_POINT = SCT_Get("YOFFSET");
	SCT_MIDDLE_POINT = SCT_Get("XOFFSET");
	SCT_aniResetAll();
end

-------------------------
--Set the font of an object
function SCT_SetFont(object)
	local outline;

	--set outline
	if (SCT_Get("FONTSHADOW") == 1) then
		outline = "";
	elseif (SCT_Get("FONTSHADOW") == 2) then
		outline = "OUTLINE";
	elseif (SCT_Get("FONTSHADOW") == 3) then
		outline = "THICKOUTLINE";
	end
	
	--set font
	object:SetFont(SCT_FONTS[SCT_Get("FONT")].path, SCT_Get("TEXTSIZE"), outline);

end

-------------------------
--Set the font of an object
function SCT_SetFontSize(object, textsize)
	local cFont, cTextsize, cOutline;
	
	--get current values
	cFont, cTextsise, cOutline = object:GetFont() 	
	
	--if size then update it
	if (cTextsize ~= textsize ) then
		object:SetFont(cFont, textsize, cOutline);
	end
end

